//gW 2010-02-17   'View'            tab=3

	/*  viewitem = *(viewtyp ":" viewvalue *("," viewvalue) ;)
	 *     
	 *		viewtyp = *("SUMMARY" / "CATEGORIES" / "NOTES" / "LOCATION") | ("DATE") 
	 *			Note:  "DESCRIPTION" has to be replaced with "summary" 
	 *             
	 *		'viewtyp' other than 'DATE' MAY occur more than once;
	 * 	for each 'viewtyp' one 'viewvalue' MUST occur at least once, 
	 *		but MAY occur more than once
	 * 
	 *		For each reminder the items are checked against the 'viewitem'. 
	 * 	If the condition is met, the reminder is included in the 'View' list.
	 * 	 
	 *    For 'viewtyp' = "DATE"  valid entries to 'viewvalue' are:
	 * 		'viewvalue' = *('TODAY'/'CURRENT_WEEK'/'NEXT_WEEK'/'NEXT_2WEEKS'/'CURRENT_MONTH' ),
	 * 		the reminder date has to fall within the 'viewvalue' range; all date ranges
	 *			are related to 'today'.
	 * 
	 *	  For 'viewtype' = *("SUMMARY" / "CATEGORIES" / "NOTES" / "LOCATION")
	 * 		the 'viewvalue' string MUST exist in the reminder item.
	 * 	
	 * 		'viewvalue' with leading '#' character will invert the condition
	 * 			the 'viewvalue' string MUST NOT exist in the reminder item.		
	 *
	 * 	Multiple 'viewvalue' for one 'viewtyp' are handled as 'OR' connected,
	 * 	multiple 'viewtyp' for one 'viewitem' are handled as 'AND' function.
	 * 	
	 *  		Example:  "DESCRIPTION:Mail,Send;CATEGORIES:business;categories:projectA;"
	 * 	 		read:  'mail' OR 'send' in 'DESCRIPTION' AND CATEGORIES='business' AND CATEGORIES='projectA'
	 *             	
	 *  	'Criteria' descriptors 
	 * 		The 'viewtyp' (called 'Criteria' at UI level) is stored to a preference:
	 * 			'extensions.reminderFox.views'
	 * 		with 'general' descriptors. At the UI those descriptors have 'local'
	 * 		equivalent stored with "reminderFox-bundle" / reminderfox.properties.
	 * 
	 * 		Example:  general: "LOCATION:Hamburg", local: "Ort:Hambug" 
	 **/

/**
     *    For 'viewtyp' = "DATE"  valid entries to 'viewvalue' are:
     *      "DATE:viewvalue1 (","viewvalue2);)
     * 
     *      The general 'viewvalueX' is:
     *      'viewvalueX' = {num}{D|M|Y}
     *          {num} is the count for days, months, or years, 
     *              if positiv date is in future, negative in the past 
     *          'D' is days, 'M' is month, 'Y' is year, only one value allowed
     *
     *          'viewvalue2' (optional)
     *          if 'viewvalue2' is obmitted 'viewvalue1' is related to 'today'
     *          if 'viewvalue1' > 'viewvalue2' the values are exchanged
     *      
     *  Special case for 'viewvalue1'   (**compatibility with previous version**)
     *    if one of the following definition matches 'viewvalue2' is ignored
     * 
     *    'viewvalue1' = *('TODAY'/'CURRENT_WEEK'/'CURRENT_MONTH'/'CURRENT_YEAR')
     *      and 
     *    'viewvalue1' = *('NEXT_WEEK'/'NEXT_2WEEKS')   
     *        these values are deprecated but supported to be compatible with
     *        prev version.
     *        Instead use new definition 'xD', eg.: 
     *           'NEXT_WEEK' -->  '7D',   'NEXT_2WEEKS' -->  '14D'
     *        or 'NEXT_WEEK' -->  '-7D,7D'  which looks into past and furture 
     *            7 days giving a span of 7days + today + 7days
     *
     * Examples with 'today' as of 'Jan 05 2010':
     *   
     *  -90D    -->Wed Oct 07 2009 00:00:00 GMT+0200 Tue Jan 05 2010 13:11:51 GMT+0100
     *  10D,40D -->Fri Jan 15 2010 00:00:00 GMT+0100 Sun Feb 14 2010 00:00:00 GMT+0100
     *  -1M,2M  -->Sat Dec 05 2009 00:00:00 GMT+0100 Fri Mar 05 2010 00:00:00 GMT+0100
     *  3M      -->Tue Jan 05 2010 13:13:59 GMT+0100 Mon Apr 05 2010 00:00:00 GMT+0200
     *  120D,1M -->Fri Feb 05 2010 00:00:00 GMT+0100 Wed May 05 2010 00:00:00 GMT+0200
     *  TODAY   -->Tue Jan 05 2010 00:00:00 GMT+0100 Tue Jan 05 2010 00:00:00 GMT+0100     
     *  CURRENT_MONTH -->Fri Jan 01 2010 00:00:00 GMT+0100 Sun Jan 31 2010 00:00:00 GMT+0100
     * 
*/
	 
	// strings to hold 'locale' descriptors for 'Criteria' and 'Date'
  var rmFxViewLocalDesc  = reminderFox_getBundle().getString("rf.html.heading.description");
  var rmFxViewLocalCat   = reminderFox_getBundle().getString("rf.add.reminders.tooltip.categories");
  var rmFxViewLocalNotes = reminderFox_getBundle().getString("rf.add.mail.message.notes");
  var rmFxViewLocalLoc   = reminderFox_getBundle().getString("rf.add.reminders.tooltip.locaton");
  var rmFxViewLocalDate  = reminderFox_getBundle().getString("rf.html.heading.date");

  var rmFxViewToday      = reminderFox_getBundle().getString("rf.views.heute"); 			//0: "Heute"			"TODAY"
  var rmFxViewCurWeek    = reminderFox_getBundle().getString("rf.views.currentweek"); 	//1: "Diese Woche"	"CURRENT_WEEK"
  var rmFxViewCurMonth   = reminderFox_getBundle().getString("rf.views.currentmonth"); //2: "Dieser Monat"  "CURRENT_MONTH"
  var rmFxViewCurYear    = reminderFox_getBundle().getString("rf.views.currentyear");  //3: "Dieses Jahr"   "CURRENT_YEAR"

  var rmFxViewNextWeek    = reminderFox_getBundle().getString("rf.views.nextweek.label"); // "Nchste Woche"  "NEXT_WEEK"
  var rmFxViewNext2Weeks  = reminderFox_getBundle().getString("rf.views.next2weeks");     // "2Wochen"        "NEXT_2WEEKS"

  var viewDateAlreadydefined =  reminderFox_getBundle().getString("rf.views.date.alreadydefined");

  var viewSortUp       = reminderFox_getBundle().getString("rf.views.sorter.viewUp"); 
  var viewSortDn       = reminderFox_getBundle().getString("rf.views.sorter.viewDown"); 
  var criteriaSortUp   = reminderFox_getBundle().getString("rf.views.sorter.criteriaUp"); 
  var criteriaSortDn   = reminderFox_getBundle().getString("rf.views.sorter.criteriaDown");  
  var validNameString  = reminderFox_getBundle().getString("rf.views.criteria.notvalidString") + ":  ; : > <";

  var newView          = reminderFox_getBundle().getString("rf.views.new");        // new
  var newViewTitel     = reminderFox_getBundle().getString("rf.views.newView");    // New 'View'
  var newViewInstruct  = reminderFox_getBundle().getString("rf.views.enterName");  // Enter the new 'View' name:

  var tttCriteria      = reminderFox_getBundle().getString("rf.views.ttt.criteria");  // "Select a 'Criteria' to edit"
  var tttEdit          = reminderFox_getBundle().getString("rf.views.ttt.edit");   //  "Edit 'View's"
  var tttEditor        = reminderFox_getBundle().getString("rf.views.ttt.editor");
  var viewExportSend   = reminderFox_getBundle().getString("rf.views.editor.exportSend");
  var viewName         = reminderFox_getBundle().getString("rf.views.name");

  var viewsChange      = reminderFox_getBundle().getString("rf.views.change");                        
  var viewsEdit        = reminderFox_getBundle().getString("rf.views.edit");
  var viewsAdd         = reminderFox_getBundle().getString("rf.views.add");

      
  var rmFxViewPrefs;
  var rmFxViews;
  var rmFxViewChanged     = false;
  var rmFxView2Save       = false;
  var gRmFxViewItems      = "";
  var gRmFxViewLabel      = "";
  var statusMsg = "";

  var gViewDateSpan = { start: null, end: null };  
    
//	 var isImportant = reminder.priority == REMINDER_PRIORITY_IMPORTANT;
//	 var isCompleted = reminderFox_isCompletedForDate( reminder, reminder.date);


// === main function to pre-select reminder based of 'VIEW' selection === 
function rmFx_ViewThis(reminder) {
  
  gViewDateSpan.start = null;
  gViewDateSpan.end = null;
 
 
  if (gRmFxViewItems == "") return true;  // no view at all !

  var statusString = true;
  var statusStringALL  = true;
        
  //  if definition doesn't has DATE item, set it TRUE
  var statusDate = (gRmFxViewItems.search(/DATE:/) == -1) ? true : false;
                    
	var viewArray = gRmFxViewItems.split(";")
	for (var i=0; i < viewArray.length ; i++) {
		var viewvalueInvert = false;	// for checking "NOT" included string

      var viewtyp = viewArray[i].split(":")[0].toLowerCase();
      var viewvalue = viewArray[i].split(":")[1];


      if (reminder[viewtyp] == null ) {
         statusStringALL = false;
      } else {

         if ((viewtyp != null) || (viewtyp != "")) {

                if (viewtyp == 'date') {    //  (" check the 'date' value of reminder");
                    rmFx_View.gDateSpan(viewvalue);

				    var limitWeeklyReminders = false;   
				    var statusDate = (reminderFox_getAllRemindersInDateRange
				            (reminder, gViewDateSpan.start, gViewDateSpan.end, limitWeeklyReminders)
				            .length > 0) ? true : false;   
                    
                } else {                    //  (" check string values");

                var statusString = false;
                				                
				var viewdetail = viewvalue.split(",")
				for (var j1=0; j1 < viewdetail.length ; j1++) {
					
					var cViewDetail = viewdetail[j1];
					if (cViewDetail[0] == "#") {
						viewvalueInvert = true;
						cViewDetail = cViewDetail.substring(1,viewvalue.length)
					}
 						if (viewvalueInvert == true) {  //  if cViewDetail NOT included
							if (reminder[viewtyp].toLowerCase().indexOf(cViewDetail.toLowerCase()) == -1){
								statusString = true;
							}
						} else {                        //  if cViewDetail IS  included													
							if (reminder[viewtyp].toLowerCase().indexOf(cViewDetail.toLowerCase()) != -1){
								statusString = true;	                                	
							}
						}
	                }	// for viewdetail
				} // string 
                statusStringALL = statusStringALL && statusString;                    
	        } //viewtype # null
        } // reminder[]
    } // viewarry
	return (statusDate && statusStringALL && statusString);
}

////// 'View'  functions ///////////////////////////////////////////////////////
function rmFx_View () { 
}


//=== VIEW - EDITOR ============================================================

// ==== part 1 -- pulldown menu with "addreminder-dialog.xul" ============

/**
 *  Builds the 'View' pull-down menu in RL Dialog for all views stored to 
 *  the .prefs
 */
rmFx_View.MenuBuild = function (thisLabel) {
	var exportView="";

   try {
      var rmFxViewPrefs = reminderFox_getUnicodePref(REMINDER_FOX_PREF + "." + "views");
   } catch (ex) {}

	var mList = document.getElementById('view_Menupopup');

	/* delete all 'View' items only */
	var count = mList.childNodes.length;
	
	while ( count-- ) {
		var child = mList.childNodes[count];	
		if (count != 0) {		// count == 0 is the first entry, the "No View"
			mList.removeChild(mList.childNodes[count]);
		}
	}
	var m1 = document.createElement("menuseparator");
	(mList.appendChild(m1));

	if ((rmFxViewPrefs != null) && (rmFxViewPrefs != "")) {

		rmFxViews = rmFxViewPrefs.split(";>,");

		var vCount = rmFxViews.length; 
	
		while ( vCount -- ) {
			var m1 = document.createElement("menuitem");
			
			var newItemA = rmFxViews[vCount].split("=<");
			
			if (newItemA[0].length > 0) {
				try {
               m1.setAttribute("type" , "checkbox");
               m1.setAttribute("autocheck" , "false");
					m1.setAttribute("oncommand", "rmFx_View.Set(this);");
					
					m1.setAttribute("label", newItemA[0]);		
					m1.setAttribute("value", newItemA[1]);
					
					// tooltiptext  displays the 'value' of this 'View'
					// need to change from 'general' to 'local' Criteria
					m1.setAttribute("tooltiptext", rmFx_View.CriteriaExchange(newItemA[1]));

					if (newItemA[0].toLowerCase() == gRmFxViewLabel.toLowerCase()) {
							m1.setAttribute("checked",true);
							exportView = newItemA[0];
					}
					(mList.insertBefore(m1,mList.childNodes[2])); // after node 1 (menuseparator)
				} catch (e) {
					alert ("VIEW: here we have a problem to add an item!");
					return;
				}
			}
		}
		var m1 = document.createElement("menuseparator");
		(mList.appendChild(m1));
	}	
	
	var m1 = document.createElement("menuitem");
	m1.setAttribute("oncommand", "rmFx_View.Manage(this);");
	m1.setAttribute("label", tttEditor);
	m1.setAttribute("value", "");
	m1.setAttribute("tooltiptext", tttEdit);
		
		(mList.appendChild(m1));
		
	if (exportView != "") {	
		var m1 = document.createElement("menuseparator");
		(mList.appendChild(m1));

		var m1 = document.createElement("menuitem");
		m1.setAttribute("oncommand", "rmFx_View.ExportView(this);");
		m1.setAttribute("label", viewExportSend + " [" + exportView + "]");
		m1.setAttribute("value", "");
		m1.setAttribute("tooltiptext", viewExportSend); 
			
    (mList.appendChild(m1));
	}
}


// ==== part 2 -- select a 'View'  =============

/**
 *  With 'View' selection the RL Dialog Title is set 
 *  and the RL list is re-fill matching the 'View' criteria
 */
rmFx_View.Set = function (thisLabel) {

  if (thisLabel != null) {
  	
  	   if (thisLabel.hasAttribute('checked') == true) {
  	   	thisLabel.removeAttribute("checked");

	      gRmFxViewLabel = "";
	      gRmFxViewItems = "";    	   	
  	   } else {
         thisLabel.setAttribute("checked", "true");
         
	      gRmFxViewLabel = thisLabel.label;
	      gRmFxViewItems = thisLabel.value;
  	   }	
  } else {
      rmFx_View.PrefLoadCurrent();
  }
  rmFx_View.SetTitel();

  refillLists();
  refillLists();  //gWTEST //gW_getStartAndEndDates
}

/**
 *  Set/Reset the "RL Dialog" Title with the 'currentview' name
 */
rmFx_View.SetTitel = function () {
	var rmFxTitel = reminderFox_getBundle().getString("rf.title");  //use .properties 

	if (gRmFxViewItems == "") {  // 	no 'View' 
		document.getElementById("view_MenuBuild").setAttribute("status", "off");
		document.getElementById("view_MenuBuild").setAttribute("tooltiptext", tttEditor);
		document.title = rmFxTitel + "";

		//ViewCurrent
		reminderFox_setUnicodePref("viewcurrent", "");	
	} else {							 // 	a 'View' was selected        
		document.getElementById("view_MenuBuild").setAttribute("status", "on");
		document.getElementById("view_MenuBuild").setAttribute("tooltiptext",  
		      rmFx_View.CriteriaExchange(gRmFxViewItems)); // ttt with 'locale'
		document.title = rmFxTitel + "    " + viewName 
				+ ": [" + gRmFxViewLabel + "]";
		//ViewCurrent
		reminderFox_setUnicodePref("viewcurrent", gRmFxViewLabel + "=<" + gRmFxViewItems + ";>,");
	}
	//  rmFx_View4tabs();  //gWtodo +++2todo +++test ++++   
}

/**
 *  rmFx_View4tabs  *** under development ***    //gW_View4TABs +++2todo +++test
 * 
 *     plan:  to show all 'todo'/'lists with VTODO'  in just ONE RmFx TAB
 *  
 */
function rmFx_View4tabs() {
	
		// reminders:new,ToDo's:sendTest,Geburtstage:dieserMonat
	
	/*   store 'View' pro TAB
	 *
 	 *	viewcurrent= (tabname ":" viewlabel) *(,tabname ":" viewlabel)
	 *  
	 * 	Check with each call if all TABS all valid (non deleted), if TAB missing,
	 *  also delete that definition from this string  
	 */
	 
	// extensions.reminderFox.todoLists = followup,ToDo's    'reminders' not stored to prefs 
	
	var namesTabs = reminderFox_getPreferenceValue( REMINDER_FOX_TODO_LISTS )
			+ ",Reminders,ToDo's";
	var nameList = namesTabs.split(",");

	
	var viewCurrentArray = new Array(); 

	try {
		var viewCurrent = reminderFox_getUnicodePref(REMINDER_FOX_PREF + "." + "viewcurrent1");
		if (viewCurrent != "") viewCurrentArray = viewCurrent.split(",");
	} catch(e) {}	
	
	var tabSelected =	document.getElementById("tabList").selectedItem.label;

	var newViewList = new Array();
	
  	for ( var n in viewCurrentArray ) {

  		var current = viewCurrentArray[n];
  		
  		if (current.indexOf(':') != -1) {
			var currentTab = current.split(':')[0];
 			var currentView = current.split(':')[1];
  		}  else {
  			// this entry is without the 'tab' ... have to migrate
  			// and use it as 'Reminders'
			var currentTab = 'Reminders';
 			var currentView = current;
  		}
  		
  		if (currentTab != tabSelected)		/*  selected tab */
  	/*		&& (gRmFxViewItems != "")	*/		/*  'No View' selected */
  		{ 
  			newViewList[newViewList.length] = currentTab + ":" + currentView;	
  		}
  	}
  	if (gRmFxViewItems != "") {
		newViewList[newViewList.length] = tabSelected + ":" + gRmFxViewLabel;	
  	}
  	var newViewDef = newViewList.join(',');
 
	reminderFox_setUnicodePref("viewcurrent1", newViewDef);
	
}
//gW_View4TABs +++2todo +++test +++test +++test +++test +++test +++test




// === part 3 -- Manage the 'View's  =============
/**
 *   call 'rmFx_View.ManageLoad()' to load the defined 'View's from the pref 
 */	
rmFx_View.Manage = function () {
	window.openDialog("chrome://reminderfox/content/categories/rmFxViewEditor.xul",
			"rmFx_View.ManageDialog", "chrome,resizable,modal" );
}

/**
 *  build the <menupopup id='criteria_Labels'> from .prefs entiry;
 *  'value'  is used to work on internal string as stored with the .views prefs
 */
rmFx_View.ManageLoad = function (sortMode) {
		var view_List = document.getElementById('view_List');
		var criteria_List = document.getElementById('criteria_List');
		var criteria_Labels = document.getElementById('criteria_Labels');
		
		/* delete all lists and menu   */
		rmFx_View.clearThis(view_List);
		rmFx_View.clearThis(criteria_List);
		rmFx_View.clearThis(criteria_Labels);

		// add menuitems from .prop to ensure we use the same with XUL and js 
		rmFx_View.MenuItems (criteria_Labels, rmFxViewLocalDesc);  //  "SUMMARY");
		rmFx_View.MenuItems (criteria_Labels, rmFxViewLocalCat);   //  "CATEGORIES");
		rmFx_View.MenuItems (criteria_Labels, rmFxViewLocalNotes); //  "NOTES");
		rmFx_View.MenuItems (criteria_Labels, rmFxViewLocalLoc);   //  "LOCATION");
      rmFx_View.MenuItems (criteria_Labels, rmFxViewLocalDate);  //  "DATE");
    
	// ("--- load the 'View's from .prefs to 'view_List' ---"); 
	try { // gW UnicodePref
		rmFxViewPrefs = reminderFox_getUnicodePref(REMINDER_FOX_PREF + "." + "views"); 
		rmFxViews = rmFxViewPrefs.split(";>,")
		
		var view_List = document.getElementById('view_List');

		var vCount = rmFxViews.length;
		while ( vCount -- ) {
			var m1 = document.createElement("listitem");
			// apply parameters & insert before node 0
			if (rmFxViews[vCount] != "") {
				
				var xx =rmFxViews[vCount].split("=<");	// ex: FoxInfo=<DATE:TODAY;>,
				m1.setAttribute("label", xx[0]);		// FoxInfo
				m1.setAttribute("value", xx[1]);		// DATE:TODAY	

				m1.setAttribute("oncommand", "rmFx_View.ViewFocus(this);");
				m1.setAttribute("tooltiptext", 
				  rmFx_View.CriteriaExchange(xx[1])); 

				if ( view_List.childNodes.length == 0 ) {
					 view_List.insertBefore(m1,null);
				}
				else {
					view_List.insertBefore(m1,view_List.childNodes[0]);
				}
			}
		}
	} catch (s) {};
	rmFx_ViewEdit.Start(true /*vmRemove*/);
}


/**
 *  Select a 'View' on the 'view_List' and write it's
 *  'criteria's to the 'criteria_List' 
 */
rmFx_View.ViewFocus = function (mode) {
	
   // for the sorting icon set focus and ttt
   if (mode == 'view') {
      document.getElementById('sorter_Box').setAttribute("type", mode);       
		document.getElementById('view_SorterUp').tooltipText = viewSortUp;
      document.getElementById('view_SorterDown').tooltipText = viewSortDn;
   } 
   if (mode == 'criteria') {
      document.getElementById('sorter_Box').setAttribute("type", mode);
		document.getElementById('view_SorterUp').tooltipText = criteriaSortUp;
		document.getElementById('view_SorterDown').tooltipText = criteriaSortDn;
   }

   // get the 'criteria' for selected 'view'
   var view_List= document.getElementById('view_List')
 
   var vLabel = view_List.currentItem.label;		// FoxInfo
	var vValue = view_List.currentItem.value;		// DATE:TODAY
		
   var aValues = vValue.split(";")


   // remove currently displayed 'criteria' and rebuild with new
   var criteria_List = document.getElementById('criteria_List');
   
   rmFx_View.clearThis(criteria_List);
	for (var i = 0; i < aValues.length; i++){	
      var m1 = document.createElement("listitem");	
			m1.setAttribute("label", rmFx_View.CriteriaExchange(aValues[i]));
			m1.setAttribute("tooltiptext", tttCriteria);		
		criteria_List.appendChild(m1);
	}
	
	
	
	rmFx_ViewEdit.Start(false /*vmRemove*/);
}

/**
 *   Add a new 'View' using the 'view_Add' button
 */
rmFx_View.ViewAdd = function (mode) {
	var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                  .getService(Components.interfaces.nsIPromptService);

	var check = {value: false};      // default the checkbox to false
	var input = {value: newView};    // default the edit field to 'new'
	if (prompts.prompt(null, newViewTitel, newViewInstruct, input, null, check) == false) {return;};
	
	// OK pressed, input.value holds the value
	rmFxView2Save = true;  // set to save any changes with close
	
	var view_List = document.getElementById('view_List'); 
	
	// add this new 'View' ... don't allow ':', ',', ';', '>', '<' in name of viewList	
	if (!rmFx_View.nameValid(input.value, view_List)) return; // noDups

	var m1 = document.createElement("listitem");	
		m1.setAttribute("label", input.value);
		m1.setAttribute("value", "SUMMARY:??");
		m1.setAttribute("tooltiptext", "SUMMARY:??");
	view_List.appendChild(m1);
	
	view_List.selectedIndex = view_List.childNodes.length -1;
		
   // add to Criteria_List 	
   var criteria_List = document.getElementById('criteria_List');
	rmFx_View.clearThis(criteria_List);
	
	var m1 = document.createElement("listitem");	
		m1.setAttribute("label", (rmFxViewLocalDesc + ":??"));
		m1.setAttribute("tooltiptext", tttCriteria);
	criteria_List.appendChild(m1);

   rmFxUtil.dump2Console ('viewModes', "view_List.selectedIndex:" 
         + (view_List.childNodes.length -1)); 
		
	sizeToContent();
}


/**
 * Remove a 'View' from 'view_List' using 'view_Remove' button
 */
rmFx_View.ViewRemove = function (mode) {

	var view_List = document.getElementById('view_List');
	var i = view_List.selectedIndex
	if (i == -1) {return};
	
	var viewLabel = view_List.childNodes[i].label

 	// prompt the user before Deleting
	var title =	reminderFox_getBundle().getString("rf.views.ttt.viewremoveTitel");
	var msg =	reminderFox_getBundle().getString("rf.views.ttt.viewremove1")
					+ "  >" + viewLabel + "<  " 
					+ reminderFox_getBundle().getString("rf.views.ttt.viewremove2");
	var key0 =	reminderFox_getBundle().getString("rf.button.ok");
	var key1 =	reminderFox_getBundle().getString("rf.button.cancel");	
	
	if (rmFxUtil.ConfirmEx(title, msg, key0, key1) == 1 ) {  // cancel pressed
			return;
	}			
	// buttonPressed  OK ... delete

	rmFxView2Save = true;
	
	var criteria_List = document.getElementById('criteria_List');
   rmFx_View.clearThis(criteria_List);

	view_List.removeChild(view_List.childNodes[i])
	
	rmFx_ViewEdit.Start(true /*vmRemove*/);
}


// === part 4 -- manage Criteria ====================
/**
 * Select 'criteria' in 'criteria_List' to store it to 'criteria_Vbox'
 */ 
rmFx_View.CriteriaToEdit = function (mode) {

	if (mode.selectedIndex == -1 ) {return};
	
	// set edit 'criteria'
	document.getElementById('sorter_Box').setAttribute("type", "criteria");
		
   document.getElementById('view_SorterUp').tooltipText =   criteriaSortUp;
   document.getElementById('view_SorterDown').tooltipText = criteriaSortDn;
	
	var criteriaString = mode.selectedItem.label;
	var thisCritera = criteriaString.split(":")[0];
	var thisValue = criteriaString.split(":")[1];
	
   // select the menu item from pulldownmenu 'criteria_Typs'
	var criteria_Typs = document.getElementById('criteria_Typs');	
	for (var i=0; i < criteria_Typs.childNodes[0].childNodes.length; i++ ) { 
		if (thisCritera.toLowerCase() == criteria_Typs.childNodes[0].childNodes[i].label.toLowerCase()) {
				criteria_Typs.selectedIndex = i;
		}
	}
	rmFx_ViewEdit.CriteriaEdit(thisCritera /*cType*/, thisValue /*cValue*/);
}


/**
 * Read the 'criteria_Vbox' string(s) and replace/add to 'criteria_List'
 * 
 */
rmFx_View.CriteriaEditEnter = function () {

	var view_List = document.getElementById('view_List').selectedIndex;

	var criteria_Typs  = document.getElementById('criteria_Typs').label
	var generalTyp = rmFx_View.Criteria2General(criteria_Typs).toUpperCase();

   rmFxUtil.dump2Console ('viewCriteria', "CriteriaEditEnter  criteria_Typs::" 
            + criteria_Typs + "   generalTyp:" + generalTyp); 
    
   switch (generalTyp) {
   case "DATE": {
      rmFxViewChanged = true;

	   // 'DATE' needs different code here for 'Panel'                          //gWViewDATE
	   if (rmFx_View.hasPanel()) { //   with 'panel'  supported
	        
	      var newString = "";
	      var eStringBegin = document.getElementById('dateBoxBegin').value 
	      var eStringEnd   = document.getElementById('dateBoxEnd').value;
		
	    /* 'old'/deprecated strings not allow for second/end value
	     *   need the 'locale' strings,  check/convert  strings to 'general'     
	     */
	      eStringBegin = rmFx_View.Criteria2General(eStringBegin);
	      eStringEnd   = rmFx_View.Criteria2General(eStringEnd);    
		    
	      if (eStringBegin.search(/NEXT_WEEK|NEXT_2WEEKS/) > -1)  eStringEnd = "";
	
	      // check second string, if empty or 0 (null)
	      if ((eStringEnd == "") 
	         || (parseInt(eStringEnd.substring(0,eStringEnd.length-1)) == 0))  {
	      newString = eStringBegin;
				eStringBegin = "";
				eStringEnd = "";            
	      } else {
				// if 'eStringBegin' has a zero value, set eStringBegin == eStringEnd and erase eStringBegin
				if (parseInt(eStringBegin.substring(0,eStringBegin.length-1)) == 0) {
					newString = eStringEnd;
					eStringBegin = "";
					eStringEnd = "";
				}
				// begin & end (dateSpan)  .. so sort it
				if (eStringEnd != "") { 
					if (rmFx_View.DateSpanFromString(eStringBegin) > rmFx_View.DateSpanFromString(eStringEnd)) {
						var temp = eStringBegin;
						var eStringBegin = eStringEnd;
						eStringEnd = temp;
					}
				   newString = eStringBegin + "," + eStringEnd;
				}
	      }
	      switch (newString) {
	            case '0D':  { newString = 'TODAY'; break;}
	            case '0W':  { newString = 'CURRENT_WEEK'; break;}
	            case '0M':  { newString = 'CURRENT_MONTH'; break;}
	            case '0Y':  { newString = 'CURRENT_YEAR'; break;}
	      }       
	                
	   } else { // no 'panel'  .. TB2/FX2 code !!  DATE with pulldown for items	      
         var dateString = document.getElementById('criteria_DateMenu').label;
	      var newString =rmFx_View.Criteria2General(dateString);
	   }
	   
      var newCriteria = generalTyp + ":" + newString;    // "Datum:-1M,10D" 
      rmFxUtil.dump2Console ('viewCriteria', "CriteriaEditEnter  DATE::" + newCriteria); 
      break;
	}  // case "DATE":
	
	default:  { // string op  
      if (generalTyp == 'CATEGORIES') {
         var newString = document.getElementById('inputRmCategories').value;
      } else {
            var newString = document.getElementById('criteria_Box').value;
      }
      if (newString != "" ) {	 // check for valid characters 
         if (!rmFx_View.nameValid(newString)) return; // noDups

         rmFxViewChanged = true;                 
         var newCriteria  = generalTyp + ":" + newString;  // "Notiz:xxyyxx"
      }
      rmFxUtil.dump2Console ('viewCriteria', "CriteriaEditEnter  generalSTRING::" + newCriteria); 

   } // default string op  
   } // switch generalTyp

	if (!rmFxViewChanged) { rmFx_ViewEdit.CriteriaEnter("-"); return;}
	
	
	// the 'criteria' changed, so store it to current 'view_List' item 
	// and reload 'criteria_List' to get 'locale' values
	
	var criteria_List = document.getElementById('criteria_List');
	var sCriteria_List = criteria_List.selectedIndex;

	var m1 = document.createElement("listitem");	
		m1.setAttribute("label", newCriteria);		     // 'local'
		m1.setAttribute("tooltiptext", tttCriteria);   //	"Select a 'Criteria' to edit"

	if (sCriteria_List == -1) { // this is a ADD because no item was selected in 'criteria_List'			
		criteria_List.appendChild(m1);
		
		var xCount = criteria_List.childNodes.length
		if ((xCount == 2) && (criteria_List.childNodes[0].label == "")) {
			criteria_List.removeItemAt(0);
			criteria_List.selectedIndex = 0;
		}
	} else {       // replace the current item
		var oldNode = criteria_List.childNodes[sCriteria_List];
		criteria_List.replaceChild(m1, oldNode);
	}

	// now change the 'value' of the 'View' 
	for (var i=0; i < criteria_List.childNodes.length; i++) {
		if (i==0) {
			var criteria_ListString = criteria_List.childNodes[0].label;
		} else {
			criteria_ListString += ";" + criteria_List.childNodes[i].label;
		}
	}
   document.getElementById('view_List').childNodes[view_List].value = criteria_ListString;
   document.getElementById('view_List').childNodes[view_List].tooltipText = criteria_ListString;
	
	rmFxUtil.dump2Console ('viewCriteria', "CriteriaEditEnter  rmFxViewChanged::" + criteria_ListString); 
	
   rmFxViewChanged = false;      
   rmFxView2Save = true;      
   rmFx_View.ViewFocus ('criteria');
}

/**
 *   ADD a new 'criteria' to the 'criteria_List' for the selected 'View'
 */
rmFx_View.CriteriaAdd = function (mode) {
    document.getElementById("criteria_List").clearSelection();	
    
	rmFx_ViewEdit.CriteriaAdd(mode);
}

/**
 *   REMOVE a 'criteria' in 'criteria_List' for the selected 'View'
 */
rmFx_View.CriteriaRemove = function (mode) {
	var aa= document.getElementById('criteria_List')
	try {
		aa.removeChild(aa.childNodes[aa.selectedIndex])
		document.getElementById('criteria_Box').value = "";
		
	// now change the 'value'
		var criteria_ListString ="";
		for (var i=0; i < aa.childNodes.length; i++) {
			if (i==0) {
				var criteria_ListString =aa.childNodes[0].label;
			} else {
				criteria_ListString += ";" + aa.childNodes[i].label;
			}
		}
		i = document.getElementById('view_List').selectedIndex
		document.getElementById('view_List').childNodes[i].value = criteria_ListString
	
		rmFxViewChanged = true;
	} catch (e) {}
	rmFx_ViewEdit.CriteriaEnter(mode);
}

/**
 *    With 'criteria_Typs' changed, check for double 'DATE'
 *    and change to 'ADD' mode
 */
rmFx_View.CriteriaTypChanged = function (event){
   
   document.getElementById("criteria_List").clearSelection();

   var locType = event.target.label;
   var generalTyp = rmFx_View.Criteria2General(locType);
    
   if (generalTyp == 'DATE' )  {                                           //gWViewDATE
        // more than one 'DATE' item per 'view' doesn't makes sense
		var criteria_List = document.getElementById('criteria_List');
		for (var i=0; i < criteria_List.childNodes.length; i++) {
         if (criteria_List.childNodes[i].label.search(rmFxViewLocalDate + ":") == 0) {
            document.getElementById('criteria_Typs').selectedIndex = 0;
            statusMsg = viewDateAlreadydefined;
            rmFxCore.statusSet (statusMsg);
            return;
         }
		}   
   }
   rmFx_ViewEdit.CriteriaAdd (generalTyp, "" /*cString*/);
}

// ---- 'DATE' panel functions    ------------------   //gWViewDATE
/**
 *  
 * Open 'panel' to let the user change the 'Date' values
 *      for 'viewvalueX' = {num}{D|W|M|Y}
 *      and optional the old values like 'TODAY'
 *
 *   @param  anchor:    object for pointing the panel to begin/end
 *   with anchor.value: a 'old' string or  'new' values ({num}{D|M|Y})
 *   @param  spanText:  holds the title text for the opened panel  
 */
rmFx_View.CriteriaDatePanel = function (anchor, spanText) {
     
   var dateString = anchor.value;  
   if ((dateString == "") || (dateString == "")) dateString ="0D";  // set 'today'

   var periode = dateString.substr(-1);
   if (periode.search(/D|W|M|Y/) == -1) return 0; 
  
   var iValue = parseInt(dateString.substring(0,dateString.length-1));
   var I = 0;
   switch (periode) {
        case 'D': I = 0; break;
        case 'W': I = 1; break;
        case 'M': I = 2; break;
        case 'Y': I = 3; break;
   }
   document.getElementById("spanPeriod").selectedIndex = I; 
   document.getElementById("dateGroup").selectedIndex = (iValue < 0) ? 0 : 1; 
   document.getElementById("dateValue").value = Math.abs(iValue);
    
   // 'anchor.id' holds a string for "begin/end" indication and set for an
   // attribute 'timeType' to have it with closing the panel
   document.getElementById("criteria_DateEdit_Titel").attributes["timeType"].value=anchor.id; 
   document.getElementById("criteria_DateEdit_Titel").value = spanText;
  
   // use 'panel' for DATE entry/edit                                   //gWViewDATE
      var panel = document.getElementById("criteria_DateEdit");   
    	panel.removeAttribute("hidden");
      panel.openPopup(anchor, 'after_start', -1, -1); 
}

/**
 *   Read the panel 'criteria_DateEdit' settings and write back to the
 *   "timeType" textbox ('dateBoxBegin' or 'dateBoxEnd') 
 */
rmFx_View.CriteriaDatePanelClose = function (xEvent) {
    
    // 'timeType'  is one of the two textboxes for DATE
    var timeType = document.getElementById("criteria_DateEdit_Titel").attributes["timeType"].value;
    
    var dValue = parseInt (document.getElementById("dateValue").value);
    var future = parseInt (document.getElementById("dateGroup").selectedIndex);
    var periode = document.getElementById("spanPeriod").selectedIndex;
    
    //    add 'W' for week 
    var spanString = ((future == 0) ? "-" : "") 
          + dValue 
          + ((periode == 0) ? "D" : 
            ((periode == 1) ? "W" : 
            ((periode == 2) ? "M" : "Y")));

    rmFxUtil.dump2Console ('CriteriaDatePanelClose', "rmFx_View.CriteriaDatePanelClose" 
            + " future:" + future
            + " dValue:" + dValue
            + " period:" + periode
            + "\n spanString:" + spanString + "  timeType" + timeType
            + "\n" + rmFxUtil.localeDate(rmFx_View.DateSpanFromString(spanString)));

    document.getElementById(timeType).value = spanString;
    document.getElementById(timeType).tooltipText 
        = rmFxUtil.localeDate(rmFx_View.DateSpanFromString(spanString));
}


// ===  Support functions   ====================================================


   rmFx_View.hasPanel = function () {
      var panel = document.getElementById("criteria_DateEdit");   
      return (panel.accessibleType != null)
   }

   rmFx_View.nameValid = function (thisName, knowNames) {
	   if (thisName.search(/(:|;|>|<)/g) != -1) {
	      rmFxCore.statusSet (validNameString);
	      return false;  // not allowed
	   }
	   // if 'knownNames' check
      if (knowNames) {
         for (var i=0; i < knowNames.children.length; i++) {
            if (thisName == knowNames.children[i].label) {
               rmFxCore.statusSet (validNameString);
            	return false;
            }
         }     
	   }
	   return true;
   }
   

/**
 *   With TAB change the 'rmView_Box' is cloned to new TAB
 *   ( called from addReminderDialog.js)
 */
rmFx_View.Clone = function () { 
    var rmViewHbox = document.getElementById('rmViewHbox');
    if (isReminderTabSelected()) { /* --- Reminders selected --- */   
            var clonedBox = rmViewHbox.cloneNode( true );
            rmViewHbox.parentNode.removeChild( rmViewHbox );
            document.getElementById("rmView_Box").appendChild( clonedBox );
    } else {  /* ---    Todos oder user-Lists selected  --- */
        /*   load XUL new */
            var clonedBox = rmViewHbox.cloneNode( true );
            rmViewHbox.parentNode.removeChild( rmViewHbox );
            document.getElementById("todoView_Box").appendChild(clonedBox);
    }
}


/**
 *   The pref REMINDER_FOX_PREF + "." + "viewcurrent"
 *   holds the current used string,
 *   if 'No View' it's empty
 * 
 *   Sets also the globals gRmFxViewItems, gRmFxViewLabel
 */
rmFx_View.PrefLoadCurrent = function () {

    gRmFxViewItems = "";
    gRmFxViewLabel = "";
     
    var rmFxViewString = "";
        
    try {
        rmFxViewString = reminderFox_getUnicodePref(REMINDER_FOX_PREF + "." + "viewcurrent");
    } catch(e) {}
    
    gViewDateSpan.start = null;
    gViewDateSpan.end = null;
       
    if (rmFxViewString != "") {
        gRmFxViewLabel = rmFxViewString.split("=<")[0];
        gRmFxViewItems = rmFxViewString.split("=<")[1].split(";>")[0];
        
        var datePos = gRmFxViewItems.search(/DATE:/);
        if (datePos != -1) {

	        var dateString = rmFxViewString.split("DATE:")[1].split(";")[0];
	  
	        rmFx_View.gDateSpan(dateString);
        }  
    }
    rmFxUtil.dump2Console ('viewPrefs', "'View' PrefsStr  load:" 
		    + "\ngRmFxViewLabel:" + gRmFxViewLabel + "  gRmFxViewItems:" + gRmFxViewItems
		    + "\nstart/end dates:" + gViewDateSpan.start + " " + gViewDateSpan.end);   
}

/**
 *   'prefs' build with Views/Criteria 
 */
rmFx_View.PrefBuildStr = function () {
    var view_List = document.getElementById('view_List');

    var rmFxViewPrefs = "";
    for (var i=0; i< view_List.childNodes.length; i++) {
        var iLabel = view_List.childNodes[i].label;        
            
        var iValues = view_List.childNodes[i].value.split(";");
        for (var j = 0; j < iValues.length; j++) {
            var thisLabel = rmFx_View.Criteria2General(iValues[j].split(":")[0]);
            var thisValues = iValues[j].split(":")[1];
            iValues[j] = thisLabel + ":" + thisValues;
        }
        rmFxViewPrefs += iLabel + "=<" + iValues.join(";") + ";>,";
       
    }
    rmFxUtil.dump2Console ('viewPrefs', "'View' PrefsStr  build:" + rmFxViewPrefs); 
    return rmFxViewPrefs;
}


/**
 *      Convert 'dateSpan' to global start/end date values 
 */
rmFx_View.gDateSpan = function (dateSpan) {                                   //gWViewDATE

    gViewDateSpan.start = null;
    gViewDateSpan.end = null;

    var dateSpanDetail = dateSpan.split(",");
    if (dateSpanDetail.length < 1 ) return;  // no 'dateSpan', clear start/end dates
    
    // --- searchFor : see 'addReminderDialog.js  at function getStartAndEndDates
    var dateType = -1;
    switch (dateSpanDetail[0].toUpperCase()) {
        case 'TODAY':
        case '0D':             {dateType = 3;  break;}
        case 'CURRENT_WEEK':
        case '0W':             {dateType = 2;  break;}
        case 'CURRENT_MONTH':
        case '0M':             {dateType = 1;  break;}
        case 'CURRENT_YEAR':
        case '0Y':             {dateType = 0;  break;}

        case 'NEXT_WEEK':      {dateType = 4; break;}
        case 'NEXT_2WEEKS':    {dateType = 5; break;}
    } 
    
    if (dateType > -1) {  // 'DATE'  has 'old' strings 
        // .. use 'addReminderDialog' function  to get the dates 
        var theseDates = getStartAndEndDates(dateType, true);
    }  
       else {           //  'DATE'  has 'new' value(s)
        var theseDates = { start: null, end: null }; 
                 
        theseDates.start = rmFx_View.DateSpanFromString(dateSpanDetail[0]);     
        if (dateSpanDetail.length > 1) { 
            theseDates.end   = rmFx_View.DateSpanFromString(dateSpanDetail[1]);
        } else {    
            theseDates.end = new Date();  //  'dateSpan2' obmitted  --> set 'today' 
        }
        // check for .start > .end --> exchange values !!
        if (theseDates.end < theseDates.start) {
            var temp = theseDates.start;
            theseDates.start = theseDates.end;
            theseDates.end = temp;
        }
    }
    gViewDateSpan.start = theseDates.start;
    gViewDateSpan.end = theseDates.end;
}

/**
 *   Convert 'dateString' string to Date() value relative to 'today'
 */
rmFx_View.DateSpanFromString = function (dateString) {
    var periode = dateString.substr(-1);
    if (periode.search(/D|W|M|Y/) == -1) return 0; 
    dateString = dateString.substring(0,dateString.length-1);
    
    var numValue = parseInt(dateString);
    
        var dD = 0;
        var dM = 0;
        var dY = 0;
        switch (periode) {
            case 'D': dD = numValue; break;
            case 'M': dM = numValue; break;
            case 'Y': dY = numValue; break;
            case 'W': dD = numValue*7; break;
        }
    var sday = new Date();          
    return new Date( sday.getYear()+ 1900 + dY, 
                     sday.getMonth() + dM, sday.getDate() +dD);
}

/**
 *    exchange  'local' and 'general' criteria descriptors 
 */
rmFx_View.CriteriaExchange = function (criteria) {        
        var xCriteria = criteria.split(";");
        for (var j=0; j < xCriteria.length; j++) {
            
            var labelString = xCriteria[j].split(":")[0];
            var valString   = xCriteria[j].split(":")[1];

            if (xCriteria[j].split(":")[0] == "DATE") {
                
                // for 'DATE' change {num}{D|M|Y} values to strings if possible    
                if (valString.search(/,/) == -1) {  // only one value
                    if (valString == '0D')  valString = rmFx_View.Criteria2Locale('TODAY');
                    if (valString == '0W')  valString = rmFx_View.Criteria2Locale('CURRENT_WEEK');
                    if (valString == '0M')  valString = rmFx_View.Criteria2Locale('CURRENT_MONTH');
                    if (valString == '0Y')  valString = rmFx_View.Criteria2Locale('CURRENT_YEAR');
                }
                valString = rmFx_View.Criteria2Locale(valString); 
            } 

            xCriteria[j] = rmFx_View.Criteria2Locale(labelString) 
                    + ":" + valString;
        } 
        return xCriteria.join(";");
}

        rmFx_View.Criteria2Locale = function (criteria) {
            switch (criteria) {
                case "SUMMARY":             {return  rmFxViewLocalDesc; }
                case "CATEGORIES":          {return  rmFxViewLocalCat; }
                case "LOCATION":            {return  rmFxViewLocalLoc; }
                case "NOTES":               {return  rmFxViewLocalNotes; }
                case "DATE":                {return  rmFxViewLocalDate; }
                
                case "TODAY":               {return rmFxViewToday;}         // :0
                case "CURRENT_WEEK":        {return rmFxViewCurWeek;}       // :1
                case "CURRENT_MONTH":       {return rmFxViewCurMonth;}      // :2
                case "CURRENT_YEAR":        {return rmFxViewCurYear;}       // :3

                case "NEXT_WEEK":           {return rmFxViewNextWeek;}      // :4
                case "NEXT_2WEEKS":         {return rmFxViewNext2Weeks;}    // :5
            }       
            return criteria;    // nothing matched, so return original criteria value
        }

        rmFx_View.Criteria2General = function (criteria) {
            switch (criteria) {        
                case rmFxViewLocalDesc:     {return  "SUMMARY"; }
                case rmFxViewLocalCat:      {return  "CATEGORIES"; }
                case rmFxViewLocalLoc:      {return  "LOCATION"; }
                case rmFxViewLocalNotes:    {return  "NOTES"; }
                case rmFxViewLocalDate:     {return  "DATE"; }

                case rmFxViewToday:          {return  "TODAY";}              // :0
                case rmFxViewCurWeek:        {return  "CURRENT_WEEK";}       // :1
                case rmFxViewCurMonth:       {return  "CURRENT_MONTH";}      // :2
                case rmFxViewCurYear:        {return  "CURRENT_YEAR";}       // :3

                case rmFxViewNextWeek:       {return  "NEXT_WEEK"; }
                case rmFxViewNext2Weeks:     {return  "NEXT_2WEEKS" };
            }       
            return criteria;    // nothing matched, so return original criteria value
        }


rmFx_View.SaveAndClose = function (mode) {

    rmFxUtil.dump2Console ('viewSaveAndClose', "rmFx_View.SaveAndClose  changed:" + rmFxViewChanged); 

	if (rmFxView2Save == false) {window.close(); return};

	var title = reminderFox_getBundle().getString("rf.views.closeTitel");	//'View' Editor - Closing";
	var msg   = reminderFox_getBundle().getString("rf.views.closeWarning");
	var key0  = reminderFox_getBundle().getString("rf.views.discard");		//YES, discard";
	var key1  = reminderFox_getBundle().getString("rf.views.noGoBack");		//NO, go back to Editor";
	var key1s  = reminderFox_getBundle().getString("rf.views.noSave");		//NO, save";

	if (mode == 'CANCEL') {  // ("--- rmFx_View.SaveAndClose w 'CANCEL'---");

		if (rmFxUtil.ConfirmEx(title, msg, key0, key1) == 1 ) {
			return;  // 'No' pressed .. return to XUL
		}			
		window.close(); // buttonPressed  'OK' ... discard all changes!!
		return;
	}
	
	if (mode == 'X') {  // ("--- the window 'X' at the header was used .. now handle the edited 'prefs' ---");
		if (rmFxUtil.ConfirmEx(title, msg, key0, key1s) == 0 ) { 
			return;  // buttonPressed  'OK' ... delete
		}			
	}
    
   // 'views' prefs string saved    
   rmFxViewPrefs = rmFx_View.PrefBuildStr();
   reminderFox_setUnicodePref("views", rmFxViewPrefs);
   rmFxUtil.dump2Console ('viewPrefs', "rmFx_View.viewSaveAndClose  prefs:" + rmFxViewPrefs); 
   window.close();
}


rmFx_View.clearThis = function (thisName) {
    var count = thisName.childNodes.length;
    while ( count-- ) {
        thisName.removeChild(thisName.childNodes[count]);
    }       
}


rmFx_View.MenuItems = function (xList, Label) {
            var m1 = document.createElement("menuitem");
            m1.setAttribute("label", Label);
            xList.appendChild(m1);
}




// ==== Button, List/Textboxes control  ========================================
function rmFx_ViewEdit () {
}
	rmFx_ViewEdit.Start = function (vmRemove) {
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		rmFx_ViewEdit.View     (false /*vmAdd*/, vmRemove /*vmRemove*/);
		rmFx_ViewEdit.Criteria (true  /*cmAdd*/, true     /*cmRemove*/);
                       
      rmFx_ViewEdit.cEdit (false /*c_List*/, true /*c_Mode*/, viewsChange /*cmLabel*/, "-" /*cType*/, "" /*cString*/);
        
      // OKbutton	
		document.getElementById('OK').removeAttribute("disabled");
	}
			
	rmFx_ViewEdit.CriteriaEdit = function (cType, cString) {
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      rmFx_ViewEdit.View     (true  /*vmAdd*/, true  /*vmRemove*/);
      rmFx_ViewEdit.Criteria (false /*cmAdd*/, false /*cmRemove*/);
         
      rmFx_ViewEdit.cEdit (false /*c_List*/, false /*c_Mode*/, viewsEdit /*cmLabel*/, cType, cString);

      // OKbutton
      document.getElementById('OK').setAttribute("disabled", true);
	}
		
	rmFx_ViewEdit.CriteriaAdd = function (cType) {
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      rmFx_ViewEdit.View     (false /*vmAdd*/, false /*vmRemove*/);
      rmFx_ViewEdit.Criteria (false /*cmAdd*/, false /*cmRemove*/);
        
      rmFx_ViewEdit.cEdit (false /*c_List*/, false /*c_Mode*/, viewsAdd /*cmLabel*/, cType /*cType*/, "" /*cString*/);
					
		// OKbutton
		document.getElementById('OK').setAttribute("disabled", true);
	}
	
	rmFx_ViewEdit.CriteriaEnter = function (cType) {
    // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		rmFx_ViewEdit.View     (false /*vmAdd*/, false /*vmRemove*/);
      rmFx_ViewEdit.Criteria (false /*cmAdd*/, true  /*cmRemove*/);

		rmFx_ViewEdit.cEdit (false /*c_List*/, true /*c_Mode*/, viewsChange /*cmLabel*/, cType, "" /*cString*/);
		
      // OKbutton		
		document.getElementById('OK').removeAttribute("disabled");
		}
		
    
      // ------------------------------------------------------------------     
		/**
		  *    Control the 'View' buttons Add and Delete
       */
      rmFx_ViewEdit.View = function (vmAdd, vmRemove) {
      // --------------------------------		
         rmFxUtil.dump2Console('viewsTest', "rmFx_ViewEdit.View   vmAdd:" 
	         + vmAdd + " vmRemove:" + vmRemove);

         document.getElementById('view_Add').setAttribute("disabled",        vmAdd);	
			document.getElementById('view_Remove').setAttribute("disabled",     vmRemove);
		}
		
	  /**
		 *    Control the 'Criteria' buttons Add and Delete
		 */
		rmFx_ViewEdit.Criteria = function (cmAdd, cmRemove) {
		// --------------------------------
			rmFxUtil.dump2Console('viewsTest', "rmFx_ViewEdit.Criteria   cmAdd:" 
	              + cmAdd + " cmRemove:" + cmRemove);
		
			document.getElementById('criteria_Add').setAttribute("disabled",    cmAdd);
			document.getElementById('criteria_Remove').setAttribute("disabled", cmRemove);
		}
        
     /**
      *   Controls the "criteria_Edit"
      */
      rmFx_ViewEdit.cEdit = function (c_List, c_Mode, cmLabel, locType, cString) {
      // --------------------------------		
	      rmFxUtil.dump2Console('viewsTest', "rmFx_ViewEdit.cEdit   c_List:" 
	            + c_List + " c_Mode:" + c_Mode + " cmLabel:"+ cmLabel + " locType:" + locType + " cString:" + cString
	            + "\n-------------------------");
               
         document.getElementById('criteria_List').setAttribute("disabled",   c_List);
						
         document.getElementById('criteria_Typs').setAttribute("disabled",   c_Mode); 
         document.getElementById('criteria_Mode').setAttribute("disabled",   c_Mode);
            
         document.getElementById('criteria_Mode').label= cmLabel;

         // disable all 'criteria_VBox'es -- will be enabled based on 'cType'
         document.getElementById('criteria_DateMenu').setAttribute('hidden', true);             
         document.getElementById('criteria_Date').setAttribute('hidden',     true);      
         document.getElementById('catHbox').setAttribute('hidden',           true);        

         var cType = rmFx_View.Criteria2General(locType);
         switch (cType.toUpperCase()) {
            case 'DATE': {	//  TB3/FX3  and TB2/FX2 code different !!               //gWViewDATE
                       	
            	if (rmFx_View.hasPanel()) {
	               // use DATE with PANEL  	
                  document.getElementById('criteria_Box').setAttribute('hidden', 'true')
                  document.getElementById('criteria_Date').removeAttribute('hidden')
										
	               var dateDetail = cString.split(",");
		                if (dateDetail.length < 1 ) return true;  // no 'Date'
						
	               var eDateBegin = dateDetail[0].toUpperCase();
		                var beginString = eDateBegin; 
		                
		                if (eDateBegin == rmFx_View.Criteria2Locale('TODAY').toUpperCase())         { beginString = "0D"; }
		                if (eDateBegin == rmFx_View.Criteria2Locale('CURRENT_WEEK').toUpperCase())  { beginString = "0W"; }
		                if (eDateBegin == rmFx_View.Criteria2Locale('CURRENT_MONTH').toUpperCase()) { beginString = "0M"; }
		                if (eDateBegin == rmFx_View.Criteria2Locale('CURRENT_YEAR').toUpperCase())  { beginString = "0Y"; }
		
	                    if (eDateBegin == rmFx_View.Criteria2Locale('NEXT_WEEK').toUpperCase())    { beginString = dateDetail[0]; }
	                    if (eDateBegin == rmFx_View.Criteria2Locale('NEXT_2WEEKS').toUpperCase())  { beginString = dateDetail[0]; }
	
	
		                document.getElementById('dateBoxBegin').value = beginString;
		                
		                // tooltiptext:  
		                if (beginString != eDateBegin) {    // for begin (dateDetail[0]) use strings like 'TODAY' 
		                	document.getElementById('dateBoxBegin').tooltipText = eDateBegin
		                } else {                            //   or take {num}{D|M|Y} for it
		                    document.getElementById('dateBoxBegin').tooltipText 
		                        = rmFxUtil.localeDate(rmFx_View.DateSpanFromString(beginString)); 
		                }
		                    
		                // tooltipText:  for end (dateDetail[1]) take {num}{D|M|Y} 
		            if (dateDetail.length > 1) {
			                    document.getElementById('dateBoxEnd').tooltipText 
			                       = rmFxUtil.localeDate(rmFx_View.DateSpanFromString(dateDetail[1])); 
			                    document.getElementById('dateBoxEnd').value = dateDetail[1]
		            } else {
								document.getElementById('dateBoxEnd').value = "";
		            }
		            
	            } else { // no 'panel'  .. TB2/FX2 code !! 
                  document.getElementById('criteria_Box').setAttribute('hidden', true)
                  document.getElementById('criteria_DateMenu').setAttribute('hidden', false)

				      /* delete all 'criteria_Date' items  */
				      var cList = document.getElementById('criteria_DateItems');
				      var count = cList.childNodes.length;
				      
				      while ( count-- ) {
				         cList.removeChild(cList.childNodes[count]);
				      }
				      // add menuitems from .prop to ensure we use the same with XUL and js 
				      rmFx_View.MenuItems (cList, rmFxViewToday);     //, "TODAY");
				      rmFx_View.MenuItems (cList, rmFxViewCurWeek);   //, "CURRENT_WEEK");
				      rmFx_View.MenuItems (cList, rmFxViewNextWeek);  //, "NEXT_WEEK");
				      rmFx_View.MenuItems (cList, rmFxViewNext2Weeks);//, "NEXT_2WEEKS");
				      rmFx_View.MenuItems (cList, rmFxViewCurMonth);  //, "CURRENT_MONTH");
	            	
		            // select 'cValue' from pull down menu 
		            var criteriaDate = document.getElementById('criteria_DateMenu')
		            for (var i=0; i < criteriaDate.childNodes[0].childNodes.length; i++ ) { 
		               if (cString == criteriaDate.childNodes[0].childNodes[i].label) {
		                     criteriaDate.selectedIndex = i;
		               }
		            } // select cValue
	            } // no 'panel'
               break;
            } // case DATE
            
            case 'CATEGORIES': {    // show the 'categories pull-down menu, delete the label
               document.getElementById('criteria_Box').setAttribute("hidden",   true);
               document.getElementById('catHbox').removeAttribute('hidden'); 
               document.getElementById('catLabel').value = "";
  
               document.getElementById('inputRmCategories').value = cString;
               break; 
            }

            case "-":  {  // show the <textbox>, but 'disable' it
               var criteraTyps = document.getElementById('criteria_Typs');
               criteraTyps.selectedIndex = 0;          

               document.getElementById('criteria_Box').removeAttribute('hidden'); 
               document.getElementById('criteria_Box').setAttribute("disabled",   true);
               break;
				}
				 
				case "+": {  // select first 'criteria_Typs' item       
                                   // and fall thru to 'default' .. because no 'break'   
					var criteraTyps = document.getElementById('criteria_Typs');
					criteraTyps.selectedIndex = 0;          
            }           
            default: {  // stringMode:  show the <textbox>, 'enable' it 
                    document.getElementById('criteria_Box').removeAttribute('hidden'); 
                    document.getElementById('criteria_Box').removeAttribute("disabled");  
                    document.getElementById('criteria_Box').value = cString; 
            }           
         }
      }


// === Sorter View/Categories ==================================================

rmFx_View.Sorter = function (mode) {
	
	var info = document.getElementById('sorter_Box').attributes["type"].value; /* 'view'|'criteria'*/
   var sortMode = mode.attributes["sorting"].value; /*sortMode 'up'|'down'*/

   var view_List = document.getElementById('view_List');
   var criteria_List = document.getElementById('criteria_List');
    
	if (info == 'view') {
	   rmFxUtil.dump2Console ('viewSorting', "View sorting :" + info +"   sortMode:" + sortMode);
	
      var prefStr = rmFx_View.PrefBuildStr();

      rmFx_View.clearThis(view_List);
      rmFx_View.clearThis(criteria_List);
  
      cViews = prefStr.split(";>,")
  
      // sorting of 'view's 
      if (sortMode != null) {
         if (sortMode == 'up') cViews = cViews.sort();
         if (sortMode == 'down') cViews = cViews.sort().reverse();
         rmFxView2Save = true;
      }

      var vCount = cViews.length;
      while ( vCount -- ) {
            var m1 = document.createElement("listitem");
            // apply parameters & insert before node 0
            if (cViews[vCount] != "") {
                
                var xx =cViews[vCount].split("=<");  // ex: FoxInfo=<DATE:TODAY;>,
                m1.setAttribute("label", xx[0]);        // FoxInfo
                m1.setAttribute("value", xx[1]);        // DATE:TODAY   
                m1.setAttribute("tooltiptext", xx[1] );

                m1.setAttribute("oncommand", "rmFx_View.ViewFocus(this);");

                if ( view_List.childNodes.length == 0 ) {
                     view_List.insertBefore(m1,null);
                }
                else {
                    view_List.insertBefore(m1,view_List.childNodes[0]);
                }
            }
        }        	
	}
   
   if (info == 'criteria') {
      var cView = view_List.selectedIndex;  	
      rmFxUtil.dump2Console ('viewSorting', "Criteria sorting for:" + cView + "   sortMode:" + sortMode);
      var criteria_List = document.getElementById('criteria_List');
      var vCount = criteria_List.childNodes.length;
	    
      var sList = new Array(vCount);
      while ( vCount -- ) {
	    	sList[vCount] = criteria_List.childNodes[vCount].label;
      }
	
      // sorting of 'view's 
      if (sortMode != null) {
         if (sortMode == 'up') sList = sList.sort();
         if (sortMode == 'down') sList = sList.sort().reverse();
         rmFxView2Save = true;
      }
	
      // write back to 'criteria_List'
      rmFx_View.clearThis(criteria_List);
	
      for (var i = 0; i < sList.length; i++){   
		   var m1 = document.createElement("listitem");    
		     m1.setAttribute("label", sList[i]);
		     m1.setAttribute("tooltiptext", tttCriteria);
		   criteria_List.appendChild(m1);
      }
	    
	    
      var aValues = sList.join(";");   
      view_List.childNodes[view_List.selectedIndex].value = aValues;  
      view_List.childNodes[view_List.selectedIndex].tooltipText = aValues;
       
      rmFxUtil.dump2Console ('viewSorting', "Criteria sorting :" + aValues + "   sortMode:" + sortMode);

  
	    rmFx_ViewEdit.CriteriaEnter ('-');
    }
}

   



// ===  'View' Export/Send =====================================================
/**
 *  To 'export' the by 'View' pre-selected events (reminders OR todos/lists) 
 *  on the displayed 'Reminderfox List' are collected and pasted to output with
 *  'rmFx_ExportOrSend'
 *  The 'View' name is added as 'category' to each reminder
 * 
 *  @return   (null, 'info', reminderTyp, reminders)
 */
rmFx_View.ExportView = function () {
	
	var reminders = new Array();
	var reminderFoxTodosArrayArg;
	var reminderTyp;
	var selectedTyp = 'info';

	// get reminders and todos model from the list window if possible - 
	// this allows you to get the latest categories even if not yet saved to 
	// the ics file yet
	
	// on 'reminders' or 'List/Todos' ??
	if (isReminderTabSelected() || isSubscribedCalendarTabSelected()){	
		reminderTyp = 'reminder';
		
		var currentReminders = getCurrentReminderList();
	
		var treeChildren = document.getElementById("treechildren");

		var x = 0;
		
		for ( var item = 0; item < treeChildren.childNodes.length; item++ ) {	
	      var selectedTreeItem = treeChildren.childNodes[item];
	      var id = selectedTreeItem.childNodes[0].getAttribute(REMINDER_ID_REF);    
			
			for (var j=0; j < currentReminders.length; j++) {
				
				if ( currentReminders[j].id == id ) {
					reminders[x] = reminderFox_cloneReminderFoxEvent(currentReminders[j]);
					
					if (reminders[x].categories == null) {
						reminders[x].categories = gRmFxViewLabel;
					} else {
						reminders[x].categories += "," + gRmFxViewLabel;
					}
					x++;
				}
			}
		}
	} else {
		reminderTyp = 'todo';
		
		var tab =    document.getElementById("tabList").selectedItem;
		var listName = tab.getAttribute( "label");
		
		reminderFoxTodosArrayArg =  rmFx_View.GetallTodos();
	//	reminderFoxTodosArrayArg =  reminderFoxListWindow.reminderFox_getReminderTodos();
	//	reminders[0] = rmFx_View.EventsView(reminderFoxTodosArrayArg[listName], reminderTyp); 

		var selectedEvents = new Array();
		var j=0;		
	  	for ( var n in reminderFoxTodosArrayArg[listName] ) {
			if (rmFx_ViewThis(reminderFoxTodosArrayArg[listName][n])) {

			  	selectedEvents[j] = reminderFox_cloneReminderFoxTodo(reminderFoxTodosArrayArg[listName][n]);	

				if (selectedEvents[j].extraInfo == null) {
					selectedEvents[j].extraInfo = REMINDERFOX_EXTENDED + "LISTID:" + listName;
				} else {
					if (selectedEvents[j].extraInfo.indexOf(REMINDERFOX_EXTENDED + "LISTID") == -1) {
						selectedEvents[j].extraInfo += "\\n" + REMINDERFOX_EXTENDED + "LISTID:" + listName;
					}
				}
				
				if (selectedEvents[j].categories == null) {
					selectedEvents[j].categories = gRmFxViewLabel;
				} else {
					selectedEvents[j].categories += "," + gRmFxViewLabel;
				}		
				j++;
			}
	  	} 
	  	reminders[0] = selectedEvents;
	}
	
	rmFx_ExportOrSend (null, 'info' /*selectedTyp*/, reminderTyp, reminders)
}

/**
 *      Read 'VEVENT' and VTODO' from array 'events'
 *      into a single array 'selectedEvents', add the 'TAB' label 
 *      as a 'X-Reminderfox' item to the output
 * 
 */
rmFx_View.EventsView = function (events, reminderTyp) {   
	var selectedEvents = new Array();
	var j=0;		
  	for ( var n in events ) {
		if (rmFx_ViewThis(events[n])) {
			if (reminderTyp == 'todo') {
		  		selectedEvents[j] = reminderFox_cloneReminderFoxTodo(events[n]);	
			} else {
		  		selectedEvents[j] = reminderFox_cloneReminderFoxEvent(events[n]);					
			}

	//gW 2009-11-17		
			if (selectedEvents[j].extraInfo == null) {
				selectedEvents[j].extraInfo = REMINDERFOX_EXTENDED + "LISTID:" + tabLabel;
			} else {
				if (selectedEvents[j].extraInfo.indexOf(REMINDERFOX_EXTENDED + "LISTID") == -1) {
					selectedEvents[j].extraInfo += "\\n" + REMINDERFOX_EXTENDED + "LISTID:" + tabLabel;
				}
			}

			
			if (selectedEvents[j].categories == null) {
				selectedEvents[j].categories = gRmFxViewLabel;
			} else {
				selectedEvents[j].categories += "," + gRmFxViewLabel;
			}		
			j++;
		}
  	} 
	return selectedEvents;
}	
	
	//reminderFox: Retrieving subscribed calendar: http://dl.erweiterungen.de/kalenderdateien/Feiertage_2006-2010_DE.ics

/**
 *    Read from displayed dialog or -- if fails -- read the stored 'todos'   
 */
rmFx_View.GetallTodos = function () {
	var reminders;
	var reminderFoxTodosArrayArg;
	var windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService();
	var windowManagerInterface = windowManager.QueryInterface(Components.interfaces.nsIWindowMediator);
	var reminderFoxListWindow = windowManagerInterface.getMostRecentWindow("window:reminderFoxEdit");	 

	 if (reminderFoxListWindow) {
	 	reminders = reminderFoxListWindow.reminderFox_getReminderEvents();
	 	reminderFoxTodosArrayArg =  reminderFoxListWindow.reminderFox_getReminderTodos();
	 }
	 else {
	 	reminders = reminderFox_getReminderEvents();
	 	reminderFoxTodosArrayArg =  reminderFox_getReminderTodos();
	 }
	 return reminderFoxTodosArrayArg;
}



// gRmFx_Messenger